@@ -24,7 +24,7 @@
 #' permutation in weighted redundancy analysis (ter Braak, 2022), but
 #' using QR-decomposition instead of ad-hoc least-squares functions.
 #'
-#' @return
+#' @returns
 #' A list with two elements with names \code{table} and \code{eigenvalues}.
 #' The \code{table} is as from \code{\link[vegan]{anova.cca}} and 
 #' \code{eigenvalues} gives the wrda eigenvalues.
@@ -64,6 +64,7 @@ anova.wrda <- function(object,
   msqr <- msdvif(object$formula, object$data, object$weights$rows, XZ = FALSE)
   Zw <- msqr$Zw 
   Xw <- msqr$Xw
+  dfpartial = msqr$qrZ$rank
   # residual predictor permutation.
   out_tes <- list()
   out_tes[[1]]  <- randperm_eX0sqrtw(Yw,Xw, Zw, sWn = sWn, 
@@ -77,43 +78,8 @@ anova.wrda <- function(object,
                           permutations = permutations, by = by, return = "all")
     }
   }
-  # what the env. variables explain of the trait-structured variation
-  ss <- c(sapply(out_tes, function(x) x$ss[1]), out_tes[[length(out_tes)]]$ss[2])
-  if (by == "axis") {
-    df <- c(rep(1, length(ss) - 1), out_tes[[length(out_tes)]]$df[2])
-    names(df) <- c(paste0("wRDA", seq_along(out_tes)), "Residual")
-  } else {
-    df <- out_tes[[length(out_tes)]]$df
-    names(df) <- c("wRDA", "Residual")
-  }
-  fraqExplained <- c(sapply(out_tes, function(x)x$ss[1]) / sum(out_tes[[1]]$ss), NA)
-  F0 <- c(sapply(out_tes, function(x)x$F0[1]), NA)
-  F.perm <- out_tes[[1]]$Fval
-  R2.perm <- out_tes[[1]]$R2.perm
-  if (length(out_tes)>1) {
-    for (k in seq_along(out_tes)[-1]) {
-      F.perm <- cbind(F.perm, out_tes[[k]]$Fval)
-      R2.perm <- cbind(R2.perm,  out_tes[[k]]$R2.perm)
-    }
-  }
-  p_val_axes1 <- c(cummax(sapply(out_tes, function(x)x$pval[1])), NA)
-  eig <- out_tes[[1]]$eig
-  names(eig)<- paste0("wRDA", seq_along(eig))
-  axsig_dcCA_sites <- data.frame(df = df, Variance = ss, 
-                                 R2 = fraqExplained, F = F0, 
-                                 "Pr(>F)" = p_val_axes1, check.names = FALSE)
-  object1 <- paste("Model:", c(object$call), "\n")
-  heading <- paste0("Permutation test for weighted reduncancy analysis\n",
-                    object1,
-                    "Residualized predictor permutation\n",
-                    howHead(attr(out_tes[[1]], "control") ))
-  f_sites <- structure(axsig_dcCA_sites, heading = heading, 
-                       control = attr(out_tes[[1]], "control"),
-                       Random.seed = attr(out_tes[[1]], "seed"),
-                       control = attr(out_tes[[1]], "control"),
-                       F.perm = F.perm,
-                       R2.perm = R2.perm,
-                       class = c("anova.cca", "anova", "data.frame"))
-  result <- list(table = f_sites, eig = eig)
+  f_sites <- fanovatable(out_tes, Nobs = N, dfpartial = dfpartial, 
+                         type = "wrda", calltext = c(object$call))
+  result <- list(table = f_sites, eigenvalues = attr(f_sites, "eig"))
   return(result)
 }
