@@ -42,10 +42,11 @@
 #' variables in the formulas gives an error ('undefined columns selected') 
 #' when the \code{\link{scores}} function is applied.
 #'
-#' @return
-#' The default returns a list of CWMs, SNCs, weights, \code{formulaTraits} and
-#' a list of data with elements \code{dataEnv} and \code{dataTraits}. When 
-#' \code{minimal_output = FALSE}, many more statistics are given that are 
+#' @returns
+#' The default returns a list of CWM, SNC, weights, \code{formulaTraits} and
+#' inertia (weighted variance explained by the traits and by the environmental
+#' variables) a list of data with elements \code{dataEnv} and \code{dataTraits}. 
+#' When  \code{minimal_output = FALSE}, some more statistics are given that are 
 #' mainly technical or recomputed when the return value is used as 
 #' \code{response} in a call to \code{\link{dc_CA}}.
 #'
@@ -94,143 +95,39 @@ fCWM_SNC <- function(response = NULL,
   # If set, formulaTraits, response, dataEnv, dataTraits are not used at all 
   # and have no efffect on the result
   call <- match.call()
-  #  check and amend: make sure there are no empty rows or columns 
-  if (any(is.na(response))) {
-    stop("The response should not have missing entries.\n")
-  }
-  if (any(response < 0)) {
-    stop("The response should not have negative values.\n")
-  }
-  if (is.null(dataTraits)) {
-    stop("dataTraits must be specified in dc_CA.\n")
-  }
-  if (!is.matrix(response)) {
-    response <- as.matrix(response)
-  }
-  id0 <- 1
-  while(length(id0)) {
-    TotR <- rowSums(response)
-    id0 <- which(TotR == 0)
-    if (length(id0)) {
-      response <- response[-id0, ]
-      dataEnv  <- dataEnv[-id0, ]
-    }
-    TotC <- colSums(response)
-    id0 <- which(TotC == 0)
-    if (length(id0)) {
-      response <- response[, -id0]
-      dataTraits <- dataTraits[-id0, ]
-    }
-  }
-  # delete columns with missing data
-  id <- rep(FALSE, ncol(dataEnv))
-  for (ii  in seq_along(id)) {
-    id[ii] <- sum(is.na(dataEnv[, ii])) == 0
-    if (!id[[ii]]) {
-      warning("variable", names(dataEnv)[ii], 
-              "has missing values and is deleted from the environmental data.\n")
-    }
-  }
-  dataEnv <- dataEnv[, id]
-  id <- rep(FALSE, ncol(dataTraits))
-  for (ii  in seq_along(id)) {
-    id[ii] <- sum(is.na(dataTraits[,ii])) == 0
-    if (!id[[ii]]) {
-      warning("variable", names(dataTraits)[ii], 
-              "has missing values and is deleted from trait data.\n")
-    }
-  }
-  dataTraits <- dataTraits[, id]
-  dataEnv <- as.data.frame(lapply(dataEnv, function(x) {
-    if (is.character(x)) x <- as.factor(x) else x 
-    return(x) 
-  }))
-  dataTraits <- as.data.frame(lapply(dataTraits, function(x) {
-    if (is.character(x)) x <- as.factor(x) else x 
-    return(x) 
-  }))
-  rownames(dataEnv) <- rownames(response)
-  rownames(dataTraits) <- colnames(response)
-  # end of check
-  if (divideBySiteTotals) {
-    response <- response / (TotR %*% t(rep(1, ncol(response))))
-  }
-  Y <- as.matrix(response) / sum(response)
-  TotR <- rowSums(Y)
-  TotC <- colSums(Y)
-  weights <- list(rows = TotR, columns = TotC) # unite sums
-  Nobs <- nrow(Y)
-  if (is.null(formulaTraits)) {
-    formulaTraits <- as.formula(paste("~", paste0(names(dataTraits),
-                                                  collapse = "+")))
-    warning("formulaTraits set to ~. in fCWMSNC.\n")
-  }
-  if (is.null(formulaEnv)) {
-    formulaEnv <- as.formula(paste("~", paste0(names(dataEnv),
-                                               collapse = "+")))
-    warning("formulaEnv set to ~. in fCWMSNC.\n")
-  }
+  out <- check_data_dc_CA(formulaEnv, formulaTraits, response, dataEnv, 
+                          dataTraits, divideBySiteTotals, call)
   # CWM and CWM ortho
-  # formula = formulaTraits; data = dataTraits; w = weights$columns
-  formulaTraits <- change_reponse(formulaTraits, "Y", dataTraits)
-  msqr <- msdvif(formulaTraits, dataTraits, weights$columns, XZ = FALSE)
-  sWn <- sqrt(weights$columns)
-  X <- msqr$Xw/sWn 
-  msd <- msqr$meansdvif
-  traits2T_ortho <- qr.R(msqr$qrX)
-  CWM2CWM_ortho <- solve(qr.R(msqr$qrX))
-  # so Q represents the orthonormalized predictors
-  CWM <- diag(1 / weights$rows) %*% Y %*% X
-  # add mean
-  CWM <- CWM + rep(1, nrow(CWM)) %*% matrix(msd[, 1], nrow = 1)
-  rownames(CWM) <- rownames(response)
-  # # check
-  if (!minimal_output) {
-    T_ortho <- qr.Q(msqr$qrX) / sWn
+  ms <- try(f_wmean(out$formulaTraits, tY = out$data$Y, out$data$dataTraits,
+                    weights=out$weights, name= "CWM"))
+  if (inherits(ms, "try-error")) {
+    stop("singular trait data. No CWMs generated.\n")    
   }
-  CWMs_orthonormal_traits <- CWM %*% CWM2CWM_ortho
-  traits_explain <- sum(CWMs_orthonormal_traits ^ 2 * weights$rows)
   # SNC and SNC ortho
-  formulaEnv <- change_reponse(formulaEnv, "Y", dataEnv)
-  msqr <- msdvif(formulaEnv, dataEnv, weights$rows, XZ = FALSE)
-  sWn <- sqrt(weights$rows)
-  X <- msqr$Xw / sWn 
-  msd <- msqr$meansdvif
-  env2T_ortho <- qr.R(msqr$qrX)
-  SNC2SNC_ortho <- solve(qr.R(msqr$qrX))
-  # so Q represents the orthonormalized predictors
-  SNC <- diag(1 / weights$columns) %*% t(Y) %*% X
-  SNC <- SNC + rep(1, nrow(SNC)) %*% matrix(msd[, 1], nrow = 1)
-  rownames(SNC) <- colnames(response)
-  SNCs_orthonormal_env <- SNC %*% SNC2SNC_ortho
-  env_explain <- sum(SNCs_orthonormal_env ^ 2 * weights$columns)
-  if (minimal_output) { 
-    out <- list(
-      CWM = CWM,
-      SNC = SNC,
-      formulaEnv = formulaEnv,
-      formulaTraits = formulaTraits, 
-      weights = weights, 
-      call = call, 
-      data = list(dataEnv = dataEnv, dataTraits = dataTraits)
-    ) 
-  } else {
-    out <- list(
-      CWM = CWM, 
-      CWMs_orthonormal_traits = CWMs_orthonormal_traits,
-      SNC = SNC, 
-      SNCs_orthonormal_env = SNCs_orthonormal_env,
-      Nobs = Nobs, 
-      traits_explain = traits_explain, 
-      formulaEnv = formulaEnv, 
-      formulaTraits = formulaTraits,
-      trans2ortho = list(CWM2CWM_ortho = CWM2CWM_ortho, 
-                         SNC2SNC_ortho = SNC2SNC_ortho),
-      T_ortho = T_ortho,
-      weights = weights, 
-      call = call, 
-      data = list(dataEnv = dataEnv, dataTraits = dataTraits)
-    )
+  mt <- try(f_wmean(out$formulaEnv, tY = t(out$data$Y), out$data$dataEnv,
+                    weights=out$weights, name= "SNC"))
+  if (inherits(mt, "try-error")) {
+    stop("singular environment data. No SNCs generated\n")   
+  } 
+  out <- list(
+    CWM = ms$wmean,
+    SNC = mt$wmean,
+    formulaEnv = out$formulaEnv,
+    formulaTraits = out$formulaTraits, 
+    inertia = c(traits_explain = ms$explained, env_explain = mt$explained),
+    weights = out$weights, 
+    call = out$call, 
+    data = out$data[-1] # remove Y from out$data
+  )
+  if (!minimal_output) { 
+    out <- c(out, list( 
+      CWMs_orthonormal_traits = ms$wmean_ortho,
+      SNCs_orthonormal_env = mt$wmean_ortho,
+      trans2ortho = list(CWM2CWM_ortho = ms$to_ortho, 
+                         SNC2SNC_ortho = mt$to_ortho),
+      T_ortho = ms$to_ortho,
+      E_ortho = mt$to_ortho
+    ))
   }
   return(out)
 }
@@ -298,7 +195,7 @@ f2_orth <- function(CWM,
   CWM2CWM_ortho <- solve(qr.R(msqr$qrX))
   colnames(CWM2CWM_ortho) <- rownames(CWM2CWM_ortho)
   if (all(rownames(CWM2CWM_ortho) %in% colnames(CWM))) {
-    CWM <- as.matrix(CWM[, rownames(CWM2CWM_ortho)])
+    CWM <- as.matrix(CWM[, rownames(CWM2CWM_ortho), drop = FALSE])
   } else {
     if (name == "CWM") {
       fname <- "formulaTraits"
@@ -308,7 +205,7 @@ f2_orth <- function(CWM,
     stop("All names generated by ", fname, " namely \n",
          paste(rownames(CWM2CWM_ortho), collapse = ","), 
          "\n should occur in response$", name, " being:\n", 
-         paste(names(CWM), collapse = ","), "\n" )
+         paste(colnames(CWM), collapse = ","), "\n" )
   }
   msd <- mean_sd_w(CWM, w = weights.rows)
   CWM <- CWM - rep(1, nrow(CWM)) %*% msd$mean
@@ -344,12 +241,30 @@ checkCWM2dc_CA <- function(object,
   }
   if ("dataTraits" %in% names(object)) {
     if (is.null(dataTraits)) {
-      object[["data"]]$dataTraits <- object$dataTraits
+      dataTraits<- as.data.frame(object$dataTraits)
+      dataTraits <- as.data.frame(lapply(X = dataTraits, FUN = function(x) {
+        if (is.character(x)) {
+          as.factor(x)
+        } else {
+          x
+        }
+      }))
+      rownames(dataTraits) <- rownames(object$dataTraits)
+      object[["data"]]$dataTraits <- dataTraits
     }
     object$dataTraits <- NULL
   }
   if ("dataEnv" %in% names(object)) {
     if (is.null(dataEnv)) {
+      dataEnv<- as.data.frame(object$dataEnv)
+      dataEnv <- as.data.frame(lapply(X = dataEnv, FUN = function(x) {
+        if (is.character(x)) {
+          as.factor(x)
+        } else {
+          x
+        }
+      }))
+      rownames(dataEnv) <- rownames(object$dataEnv)							   
       object[["data"]]$dataEnv <- object$dataEnv
     }
     object$dataEnv<-NULL
@@ -371,17 +286,14 @@ checkCWM2dc_CA <- function(object,
       warning("Supply trait data to the dc_CA function.\n")
     }
   } else { 
-    warning("With CWM as first element in response in dc_CA, the trait data",
-            "used to obtain the CWMs are best supplied as response$data$dataTraits.",
-            "Use the default dataTraits argument, which is NULL.\n")
+    warning("Trait data taken from the argument dataTraits.\n")
     object$data$dataTraits <- as.data.frame(lapply(dataTraits, function(x) {
       if (is.character(x)) x <- as.factor(x) else x
       return(x) 
     }))
   }
   # check weights
-  if (is.null(object$weights) || is.null(object$weights$columns) || 
-      is.null(object$weights$rows)) {
+  if (is.null(object$weights)) {
     # try dataTraits and dataEnv
     if (!is.null(object$data$dataTraits$weight)) {
       warning("species weights taken from dataTraits$weight.\n")
@@ -396,35 +308,42 @@ checkCWM2dc_CA <- function(object,
   }
   if (is.null(object$weights)) {
     warning("no weights supplied with response$CWM; weigths all set to 1.\n")
-    object$weights <- list(rows = rep(1 / object$Nobs, object$Nobs),
-                           columns = rep(1 / nrow(object$data$dataTraits),
-                                         nrow(object$data$dataTraits)))
+    object$weights <- list(columns = rep(1 / nrow(object$data$dataTraits),																		  
+                                         nrow(object$data$dataTraits)),
+                           rows = rep(1 / object$Nobs, object$Nobs))
   } else if (!is.list(object$weights)) {
     ll <- length(object$weights)
     if (ll == object$Nobs) {
       warning("no species weights supplied with response$CWM; ",
               "weigths all set to 1.\n")
-      object$weights <- list(rows = rep(1 / object$Nobs, object$Nobs),
-                             columns = object$weights)
+      object$weights <- list(columns = object$weights,
+                             rows = rep(1 / object$Nobs, object$Nobs))
     } else if (ll == nrow(object$data$dataTraits)) {
       warning("no site weights supplied with response$CWM; ", 
               "site weigths all set to 1.\n")
-      object$weights <- list(rows = object$weights,
-                             columns = rep(1 / nrow(object$data$dataTraits),
-                                           nrow(object$data$dataTraits)))
+      object$weights <- list(
+        columns = rep(1 / nrow(object$data$dataTraits),
+                      nrow(object$data$dataTraits)),
+        rows = object$weights)
     }
   }
-  if (is.null(object$weights$rows)) {
-    warning("no site weights supplied with response$CWM; ", 
-            "site weigths all set to 1.\n")
-    object$weights$rows <- rep(1 / object$Nobs, object$Nobs)
-  }
-  if (is.null(object$weights$columns)) {
-    warning("no species weights supplied with response$CWM; ",
-            "species weigths all set to 1.\n")
-    object$weights$columns <- rep(1 / nrow(object$data$dataTraits),
-                                  nrow(object$data$dataTraits))
+  if (!is.null(object$weights)) {
+    if (is.null(object$weights$rows)) {
+      warning("no site weights supplied with response$CWM; ", 
+              "site weigths all set to 1.\n")
+      object$weights$rows <- rep(1 / object$Nobs, object$Nobs)
+    }
+    if (is.null(object$weights$columns)) {
+      warning("no species weights supplied with response$CWM; ",
+              "species weigths all set to 1.\n")
+      object$weights$columns <- rep(1 / nrow(object$data$dataTraits),
+                                    nrow(object$data$dataTraits))
+    }
   }
+  # make sure columns is the first in the list.
+  object$weights <- list(columns = object$weights$columns, 
+                         rows = object$weights$rows)
+  object$weights <- lapply(X = object$weights, FUN = function(x) x / sum(x))
   # change ~. to names
   formulaTraits <- change_reponse(object$formulaTraits, "Y", 
                                   object$data$dataTraits)
@@ -443,3 +362,144 @@ checkCWM2dc_CA <- function(object,
   object$CWM2CWM_ortho <- CWM2ortho$CWM2CWM_ortho
   return(object)
 }
+
+#' @noRd
+#' @keywords internal
+f_wmean <- function(formulaEnv, 
+                    tY, 
+                    dataEnv, 
+                    weights, 
+                    name = "SNC") {
+  # SNC and SNC ortho or 
+  # CWM and CWM ortho if name = "CwM" from formulaTraits, Y and dataTraits
+  tot <- sum(tY)
+  formulaEnv <- change_reponse(formulaEnv, "Y", dataEnv)
+  if (name == "SNC") {
+    w <- weights$rows 
+  } else {
+    w <- weights$columns
+  }
+  msqr <- msdvif(formulaEnv, dataEnv, w, XZ = FALSE)
+  sWn <- sqrt(w)
+  X <- msqr$Xw / sWn 
+  msd <- msqr$meansdvif
+  E_ortho <- qr.Q(msqr$qrX) / sWn
+  # so Q represents the orthonormalized predictors
+  if (name == "SNC") {
+    w <- weights$columns
+  } else {
+    w <- weights$rows
+  }
+  SNC <- diag(1 / (tot * w)) %*% as.matrix(tY) %*% X
+  SNC2SNC_ortho <- try(solve(qr.R(msqr$qrX)))
+  if (inherits(SNC2SNC_ortho, "try-error")) {
+    warning("singular environment data. Env_explain not available.\n") 
+    SNC2SNC_ortho <- matrix(nrow = ncol(SNC), ncol = 1)
+  }
+  SNCs_orthonormal_env <- SNC %*% SNC2SNC_ortho
+  env_explain <- sum(SNCs_orthonormal_env ^ 2 * w)
+  SNC <- SNC + rep(1, nrow(SNC)) %*% matrix(msd[, 1], nrow = 1)
+  rownames(SNC) <- rownames(SNCs_orthonormal_env) <- rownames(tY)
+  rownames(E_ortho) <- colnames(tY)
+  res <- list(wmean = SNC, explained = env_explain,
+              wmean_ortho = SNCs_orthonormal_env, name = name, to_ortho=E_ortho)
+  return(res)
+}
+
+#' @noRd
+#' @keywords internal
+check_data_dc_CA <- function(formulaEnv, 
+                             formulaTraits,
+                             response,
+                             dataEnv, 
+                             dataTraits,
+                             divideBySiteTotals,
+                             call) {
+  #  check and amend: make sure there are no empty rows or columns
+  if (any(is.na(response))) {
+    stop("The response should not have missing entries.\n")
+  }
+  if (any(response < 0)) {
+    stop("The response should not have negative values.\n")
+  }
+  if (is.null(dataEnv)) {
+    stop("dataEnv must be specified in dc_CA.\n")
+  } else  dataEnv <- as.data.frame(dataEnv)
+  if (is.null(dataTraits)) {
+    stop("dataTraits must be specified in dc_CA.\n")
+  } else  dataTraits <- as.data.frame(dataTraits)
+  if (!is.matrix(response)) {
+    response <- as.matrix(response)
+  }
+  id0 <- 1
+  while(length(id0)) {
+    TotR <- rowSums(response)
+    id0 <- which(TotR == 0)
+    if (length(id0)) {
+      response <- response[-id0, ]
+      dataEnv  <- dataEnv[-id0, ]
+    }
+    TotC <- colSums(response)
+    id0 <- which(TotC == 0)
+    if (length(id0)) {
+      response <- response[, -id0]
+      dataTraits <- dataTraits[-id0, ]
+    }
+  }
+  # delete columns with missing data
+  id <- rep(FALSE, ncol(dataEnv))
+  for (ii  in seq_along(id)) {
+    id[ii] <- sum(is.na(dataEnv[, ii])) == 0
+    if (!id[[ii]]) {
+      warning("variable", names(dataEnv)[ii],
+              "has missing values and is deleted from the environmental data.\n")
+    }
+  }
+  dataEnv <- dataEnv[, id]
+  id <- rep(FALSE, ncol(dataTraits))
+  for (ii  in seq_along(id)) {
+    id[ii] <- sum(is.na(dataTraits[,ii])) == 0
+    if (!id[[ii]]) {
+      warning("variable", names(dataTraits)[ii],
+              "has missing values and is deleted from trait data.\n")
+    }
+  }
+  dataTraits <- dataTraits[, id]
+  dataEnv <- as.data.frame(lapply(dataEnv, function(x) {
+    if (is.character(x)) x <- as.factor(x) else x
+    return(x)
+  }))
+  dataTraits <- as.data.frame(lapply(dataTraits, function(x) {
+    if (is.character(x)) x <- as.factor(x) else x
+    return(x)
+  }))
+  rownames(dataEnv) <- rownames(response)
+  rownames(dataTraits) <- colnames(response)
+  # end of check
+  TotR <- rowSums(response)
+  if (divideBySiteTotals) {
+    response <- response / (TotR %*% t(rep(1, ncol(response))))
+    TotR <- rep(1, length(TotR))
+  }
+  TotC <- colSums(response)
+  TotR <- TotR / sum(TotR)
+  TotC <- TotC / sum(TotC)
+  weights <- list(columns = TotC, rows = TotR) # unit sums
+  Nobs <- nrow(response)
+  if (is.null(formulaTraits)) {
+    formulaTraits <- as.formula(paste("~", paste0(names(dataTraits),
+                                                  collapse = "+")))
+    warning("formulaTraits set to ~. in fCWMSNC.\n")
+  }
+  if (is.null(formulaEnv)) {
+    formulaEnv <- as.formula(paste("~", paste0(names(dataEnv),
+                                               collapse = "+")))
+    warning("formulaEnv set to ~. in fCWMSNC.\n")
+  }
+  
+  out <- list(formulaTraits = formulaTraits, formulaEnv = formulaEnv,
+              data = list(Y = response, dataEnv = dataEnv, 
+                          dataTraits = dataTraits),
+              call = call,  weights = weights, Nobs = Nobs)
+  return(out)
+}
