@@ -36,6 +36,9 @@
 #' unit weighted mean square or with 3 both are scaled symmetrically 
 #' to weighted mean squares equal to the square root of eigenvalues. Negative 
 #' values are treated as the corresponding positive ones by \code{abs(scaling)}.
+#' @param normed logical (default \code{TRUE}) giving standardized regression
+#' coefficients and biplot scores. When \code{FALSE}, (regular)
+#' regression coefficients and (unstandardized) biplot scores.
 #' @param tidy Return scores that are compatible with \code{ggplot2}: all 
 #' scores are in a single data.frame, score type is identified by factor 
 #' variable \code{score}, the names by variable \code{label}, and species 
@@ -52,7 +55,7 @@
 #' An example of \code{which_cor} is: \code{which_cor = list(traits = "SLA", 
 #' env = c("acidity", "humidity"))}.
 #' 
-#' @return A data frame if \code{tidy = TRUE}. Otherwise, a matrix if a single
+#' @returns A data frame if \code{tidy = TRUE}. Otherwise, a matrix if a single
 #' item is asked for and a named list of matrices if more than one item is 
 #' asked for. The following names can be included: 
 #' \code{c("sites", "constraints_sites", "centroids", "regression", "t_values",
@@ -121,8 +124,9 @@ scores.dcca <- function(x,
                         choices = 1:2, 
                         display = "all", 
                         scaling = "sym", 
-                        which_cor = "in model", 
+                        which_cor = "in model",
+                        normed = TRUE,
                         tidy = FALSE) {
  scores_dcca(x, choices = choices, display = display, scaling = scaling, 
-             which_cor = which_cor, tidy = tidy, ...)
+             which_cor = which_cor, normed = normed, tidy = tidy, ...)
 }
